// Arduino Nano Router Restarter II
// Written by Theron Wierenga, February 2016
// Modified for 3231 RTC, October 2023
// theronsarticles@gmail.com

#include <DS3231.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

#define CHAN1 2
#define CHAN2 3

// Create the Real Time Clock
DS3231 rtc;
// Create an LCD object
LiquidCrystal_I2C lcd(0x27,20,4);   // Also address 0x3F


bool century = false, flag = false;;
bool h12Flag;
bool pmFlag;

unsigned int last_sec = 0, bminute = 1, bhour = 2, ampm, bdayOfWeek, bdayOfMonth, bmonth, iyear;
unsigned int start_hour1 = 3, start_minute1 = 0, start_hour2 = 3, start_minute2 = 1;  // start times
unsigned int end_hour1 = 3, end_minute1 = 1, end_hour2 = 3, end_minute2 = 2;            // end times
String months[12] = {"January","February","March","April","May","June","July","August","September","October","November","December"};
String daysOfWeek[7] = {"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"};


void setup() {
  delay(4000);
  // Start the serial port
  Serial.begin(115200);
  Serial.println("Starting");
  // Set pins as outputs
  pinMode(CHAN1, OUTPUT);
  pinMode(CHAN2, OUTPUT);
  // Deactivate relays so NC will power circuits
  digitalWrite(CHAN1, HIGH);
  digitalWrite(CHAN2, HIGH);
  
  // Start the I2C interface
  Wire.begin();

  // Initialize the LCD
  lcd.init();
  // Print a message to the LCD.
  lcd.backlight();
  lcd.setCursor(0,0);
  lcd.print("Starting");
  lcd.setCursor(0,2);
  lcd.print("Channel 1 Powered   ");
  lcd.setCursor(0,3);
  lcd.print("Channel 2 Powered   ");

  // These values should be uncommented and the time and date set on
  // the first upload.
  // Once running correctly comment out these line and upload again.
  // The backup battery on the DS3231 mini board will then retain the time.
//  rtc.setClockMode(false);   // 12 hour mode = true, 24 hour = false
//  rtc.setSecond(0);
//  rtc.setMinute(30);
//  rtc.setHour(8);
//  rtc.setDoW(6);      // 1-7
//  rtc.setDate(20);     // 1-31
//  rtc.setMonth(10);   // 1-12
//  rtc.setYear(23);

}

void loop() 
{
// These are used for debugging if necessary
    Serial.print(rtc.getYear(), DEC);
    Serial.print("-");
    Serial.print(rtc.getMonth(century), DEC);
    Serial.print("-");
    Serial.print(rtc.getDate(), DEC);
    Serial.print(" ");
    Serial.print(rtc.getHour(h12Flag, pmFlag), DEC); //24-hr
    Serial.print(":");
    Serial.print(rtc.getMinute(), DEC);
    Serial.print(":");
    Serial.println(rtc.getSecond(), DEC);
    Serial.println(h12Flag);
    Serial.println(pmFlag);

    // Get the time
    last_sec = rtc.getSecond();
    bminute = rtc.getMinute();
    bhour = rtc.getHour(h12Flag, pmFlag); //24-hr
    bdayOfMonth = rtc.getDate();
    bmonth = rtc.getMonth(century);
    iyear = rtc.getYear();
    bdayOfWeek = rtc.getDoW();
    // Blink LED to show clock activity
    if (flag)
    {
        digitalWrite(13, HIGH);
    }
    else
    {
        digitalWrite(13, LOW);
    }
    flag = !flag;
    delay(250);
    Display_Time();
    Display_Date();
    actionNeeded();
  
}

// ************************************************************************************
// Display Time
void Display_Time()
{
  // For cursor first number is char in line, second number is line
  lcd.setCursor(0, 0);
  if (bhour < 10) lcd.print("0");  // Print 0 in front of 0-9 numbers
  lcd.print(bhour);
  lcd.print(":");
  if (bminute < 10) lcd.print("0");
  lcd.print(bminute);
  lcd.print(":");
  if (last_sec < 10) lcd.print("0");
  lcd.print(last_sec);
  lcd.print(" ");
}

// ************************************************************************************
// Display Date
void Display_Date()
{
    String temp, totstr;;
    int tlen = 0;
    lcd.setCursor(9, 0);
    temp = makeString(daysOfWeek[bdayOfWeek - 1], 11);
    lcd.print(temp);
    lcd.setCursor(0, 1);
    tlen = months[bmonth - 1].length() + 8;;
    if (bmonth > 9) tlen += 1;
    totstr = months[bmonth - 1];
    totstr.concat(" ");
    totstr.concat(bdayOfMonth);
    totstr.concat(", 20");
    totstr.concat(iyear);
    temp = makeString(totstr, 20);
    lcd.print(temp);
}

// ************************************************************************************
// Build Date and Day of Week Strings of specific length to fit LCD
String makeString(String str, int len)
{
    while (str.length() < len)
    {
        str = str + " ";
    }
    return str;
}


// **********************************************************************************
// Check Time and see if action needed
void actionNeeded()
{
    if ((bhour == start_hour1) && (bminute == start_minute1))
    {
        // Activate channel 1
        digitalWrite(CHAN1, LOW);
        lcd.setCursor(0, 2);
        lcd.print("Channel 1 Resetting ");
        Serial.println("Channel 1 Resetting ");
    }
    if ((bhour == start_hour2) && (bminute == start_minute2))
    {
        // Activate channel 2
         digitalWrite(CHAN2, LOW);
         lcd.setCursor(0, 3);
         lcd.print("Channel 2 Resetting ");
         Serial.println("Channel 2 Resetting ");
    }

    if ((bhour == end_hour1) && (bminute == end_minute1))
    {
        // Deactivate channel 1
        digitalWrite(CHAN1, HIGH);
        lcd.setCursor(0, 2);
        lcd.print("Channel 1 Powered   ");
        Serial.println("Channel 1 Powered ");
    }
    if ((bhour == end_hour2) && (bminute == end_minute2))
    {
        // Deactivate channel 2
        digitalWrite(CHAN2, HIGH);
        lcd.setCursor(0, 3);
        lcd.print("Channel 2 Powered   ");
        Serial.println("Channel 2 Powered ");
    }    
}
